@@ -15,9 +15,11 @@ import os
 from os.path import basename, join
 import shutil
 import unittest
+import warnings
 
 from ClusterShell.NodeSet import NodeSet
-from ClusterShell.Task import task_self, task_terminate
+from ClusterShell.Task import task_self, task_terminate, task_wait
+from ClusterShell.Task import Task, task_cleanup
 from ClusterShell.Topology import TopologyGraph
 from ClusterShell.Worker.Tree import TreeWorker, WorkerTree
 
@@ -99,6 +101,7 @@ class TEventHandler(TEventHandlerBase):
             self.ev_timedout_cnt += 1
 
 
+@unittest.skipIf(HOSTNAME == 'localhost', "does not work with hostname set to 'localhost'")
 class TreeWorkerTest(unittest.TestCase):
     """
     TreeWorkerTest: test TreeWorker
@@ -131,7 +134,10 @@ class TreeWorkerTest(unittest.TestCase):
     def test_tree_run_event_legacy(self):
         """test simple tree run with legacy EventHandler"""
         teh = TEventHandlerLegacy()
-        self.task.run('echo Lorem Ipsum', nodes=NODE_DISTANT, handler=teh)
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always")
+            self.task.run('echo Lorem Ipsum', nodes=NODE_DISTANT, handler=teh)
+            self.assertEqual(len(w), 4)
         self.assertEqual(teh.ev_start_cnt, 1)
         self.assertEqual(teh.ev_pickup_cnt, 1)
         self.assertEqual(teh.ev_read_cnt, 1)
@@ -144,7 +150,10 @@ class TreeWorkerTest(unittest.TestCase):
     def test_tree_run_event_legacy_timeout(self):
         """test simple tree run with legacy EventHandler with timeout"""
         teh = TEventHandlerLegacy()
-        self.task.run('sleep 10', nodes=NODE_DISTANT, handler=teh, timeout=0.5)
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always")
+            self.task.run('sleep 10', nodes=NODE_DISTANT, handler=teh, timeout=0.5)
+            self.assertEqual(len(w), 2)
         self.assertEqual(teh.ev_start_cnt, 1)
         self.assertEqual(teh.ev_pickup_cnt, 1)
         self.assertEqual(teh.ev_read_cnt, 0)      # nothing to read
@@ -233,12 +242,19 @@ class TreeWorkerTest(unittest.TestCase):
         self.assertEqual(teh.ev_close_cnt, 1)
         self.assertEqual(teh.last_read, b'Lorem Ipsum')
 
-    def _tree_run_write(self, target):
+    def _tree_run_write(self, target, separate_thread=False):
+        if separate_thread:
+            task = Task()
+        else:
+            task = self.task
         teh = TEventHandler()
-        worker = self.task.shell('cat', nodes=target, handler=teh)
+        worker = task.shell('cat', nodes=target, handler=teh)
         worker.write(b'Lorem Ipsum')
         worker.set_write_eof()
-        self.task.run()
+        task.run()
+        if separate_thread:
+            task_wait()
+            task_cleanup()
         self.assertEqual(teh.ev_start_cnt, 1)
         self.assertEqual(teh.ev_pickup_cnt, 1)
         self.assertEqual(teh.ev_read_cnt, 1)
@@ -265,6 +281,22 @@ class TreeWorkerTest(unittest.TestCase):
         """test tree run with write(), gateway is target, not in topology"""
         self._tree_run_write(NODE_GATEWAY)
 
+    def test_tree_run_write_distant_mt(self):
+        """test tree run with write(), distant target, separate thread"""
+        self._tree_run_write(NODE_DISTANT, separate_thread=True)
+
+    def test_tree_run_write_direct_mt(self):
+        """test tree run with write(), direct target, in topology, separate thread"""
+        self._tree_run_write(NODE_DIRECT, separate_thread=True)
+
+    def test_tree_run_write_foreign_mt(self):
+        """test tree run with write(), direct target, not in topology, separate thread"""
+        self._tree_run_write(NODE_FOREIGN, separate_thread=True)
+
+    def test_tree_run_write_gateway_mt(self):
+        """test tree run with write(), gateway is target, not in topology, separate thread"""
+        self._tree_run_write(NODE_GATEWAY, separate_thread=True)
+
     def _tree_copy_file(self, target):
         teh = TEventHandler()
         srcf = make_temp_file(b'Lorem Ipsum', 'test_tree_copy_file_src')
